import datetime

from dateutil.relativedelta import relativedelta


def time_ago(dt):
    rd = relativedelta(datetime.datetime.utcnow(), dt)
    
    attrs = ["years", "months", "days", "hours", "minutes", "seconds"]
    for attr in attrs:
        v = getattr(rd, attr)
        if v:
            if v == 1: attr = attr[:-1]
            return "%s %s ago" % (v, attr)
            break
    return 'right now'



import re

class ContentMiddleware(object):
    def __init__(self, mapping):
        self.mapping = mapping

    def __call__(self, environ, start_response):
        content = environ.get('CONTENT_TYPE', '') or environ.get('HTTP_ACCEPT', '')
        content = content.split(';')[0]
        
        for type, app in self.mapping:
            if re.match(type, content): return app(environ, start_response)



import os

from paste.script import templates

class WebskineTemplate(templates.Template):

    summary = "A Webskine blog deployed through paste.deploy"
    
    egg_plugins = ['webskine']
    _template_dir = 'paster_templates'
    use_cheetah = True

    def post(self, command, output_dir, vars):
        if command.verbose:
            print '*'*72
            print '* Run "paster serve %s/server.ini" to run' % output_dir
            print '* the Webskine on http://localhost:8080'
            print '*'*72



from setuptools import setup, find_packages
import sys, os

version = '0.1.2'

setup(name='webskine',
      version=version,
      description="A simple WSGI blog.",
      long_description="""\
Webskine is a simple and fun weblog. It uses AJAX calls -- actually, asynchronous JSON requests -- to create, edit and delete posts in place. Most of the functionality comes from Paste, Cheetah and jsonstore (on the server side) and jquery and TinyMCE (on the client side).""",
      classifiers=[], # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      keywords='wsgi blog',
      author='Roberto De Almeida',
      author_email='roberto@dealmeida.net',
      url='http://dealmeida.net/projects/webskine',
      license='MIT',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          'Paste',
          'PasteScript',
          'PasteDeploy',
          'jsonstore>=0.3a1',
          'python-dateutil',
          'Cheetah',
      ],
      entry_points="""
      # -*- Entry points: -*-
      [paste.app_factory]
      main = webskine.wsgiapp:make_app

      [paste.paster_create_template]
      webskine = webskine.template:WebskineTemplate
      """,
      )
      



import re
import os.path
import datetime
from urlparse import urljoin
from cgi import escape
import md5

from paste import httpexceptions
from paste.request import construct_url
from Cheetah.Template import Template

from jsonstore import EntryManager


def make_app(global_conf, dsn, templates,
        title='Test Webskine installation',
        entries=10,
        user='test',
        password='test'):
    """
    Create a weblog.

    Configuration should be like this::

        [app:weblob]
        use = egg:webskine
        title = Weblog title
        dsn = protocol://location 
        entries = 10
        user = username
        password = *******
    """
    from paste.auth.digest import digest_password, AuthDigestHandler

    from webskine.middleware import ContentMiddleware
    from jsonstore import JSONStore

    # Apps we're using.
    webskine = Webskine(dsn, templates, title, entries)
    jsonstore = JSONStore(dsn) 

    # Add authentication to the JSON store.
    def authfunc(environ, realm, username):
        return digest_password(realm, user, password)
    jsonstore = AuthDigestHandler(jsonstore, title, authfunc)

    # Pass application/json requests to jsonstore, all others to webskine.
    mapping = [ ('application/json', jsonstore),
                ('.*', webskine) ]

    app = ContentMiddleware(mapping)
    return app


class Webskine(object):
    def __init__(self, dsn, templates, title, entries):
        self.templates = templates
        self.title = title
        self.entries = entries

        # Get entry manager.
        self.em = EntryManager(dsn)

    def __call__(self, environ, start_response):
        path_info = environ.get('PATH_INFO', '/')

        # Parse request.
        dispatchers = [ ('/atom', lambda d: self._atom(**d)),
                        ('/(?P<id>\d+)?', lambda d: self._entry(**d)), ]
        for regexp, func in dispatchers:
            p = re.compile(regexp)
            m = p.match(path_info)
            if m:
                reqdict = m.groupdict()
                reqdict['environ'] = environ
                reqdict['start_response'] = start_response
                return func(reqdict)
        
        # Raise 404.
        raise httpexceptions.HTTPNotFound()

    def _entry(self, id, environ, start_response):
        """
        Return entry or list of entries.

        Return a single entry if id is specified. Otherwise,
        return last self.entries entries.
        """
        if id is None:  
            entries = self.em.search(size=self.entries)
            title = self.title
        else:
            try:
                entry = self.em.search(__id__=id)
                entries = [entry]
                title = entry['title']
            except KeyError:
                raise httpexceptions.HTTPNotFound()

        # Update entry with a proper id (pointing to the resource location).
        location = construct_url(environ, with_query_string=False, with_path_info=False) + '/'
        namespace = {'name': self.title, 'title': title, 'home': location, 'entries': entries}
        template = Template(file=os.path.join(self.templates, 'xhtml.tmpl'),
                searchList=[namespace], filter='EncodeUnicode')

        headers = [('Content-Encoding', 'utf-8'),
                   ('Content-Type', 'text/html')]
        start_response('200 OK', headers)
        return [str(template)] 

    def _atom(self, environ, start_response):
        """
        Return an atom feed.
        """
        entries = self.em.search(size=self.entries)

        # Update entry with a proper id (pointing to the resource location).
        location = construct_url(environ, with_query_string=False, with_path_info=False) + '/'
        namespace = {'title': self.title,
                     'feed': urljoin(location, 'atom'),
                     'updated': entries[0]['__updated__'],
                     'home': location,
                     'entries': entries}
        template = Template(file=os.path.join(self.templates, 'atom.tmpl'),
                searchList=[namespace], filter='EncodeUnicode')

        headers = [('Content-Encoding', 'utf-8'),
                   ('Content-Type', 'application/atom+xml')]
        start_response('200 OK', headers)
        return [str(template)]
